/******************************************************************************
 * INCLUDES
 */
#include <string.h>
#include "nordic_common.h"
#include "app_util.h"
#include "app_error.h"
#include "nrf_drv_gpiote.h"
#include "nrf_assert.h"

#include "hal_intpin.h"

static hal_intpin_cfg_t *             mp_intpins = NULL;           /**< Intpin configuration. */
static uint8_t                        m_intpin_count;              /**< Number of configured intpins. */

/**@brief Function for handling the GPIOTE event.
 *
 * @details Saves the current status of the intpin pins, and starts a timer. If the timer is already
 *          running, it will be restarted.
 *
 * @param[in]  low_to_high   Mask telling which pin(s) had a low to high transition.
 * @param[in]  high_to_low   Mask telling which pin(s) had a high to low transition.
 */
static void gpiote_event_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    uint8_t i;
 
    STATIC_ASSERT(sizeof(void *) == sizeof(uint32_t));
    
    // Pushed intpin(s) detected, execute intpin handler(s).
    for (i = 0; i < m_intpin_count; i++)
    {
        hal_intpin_cfg_t * p_btn = &mp_intpins[i];
        if (p_btn->pin_no == pin)
        {
            if (action == NRF_GPIOTE_POLARITY_HITOLO)
            {
                if (p_btn->trigger_edge == HAL_INTPIN_EDGE_HI2LO)
                {
                    p_btn->intpin_handler(p_btn->pin_no, HAL_INTPIN_EDGE_HI2LO);
                }
            }
            else if (action == NRF_GPIOTE_POLARITY_LOTOHI)
            {
                if (p_btn->trigger_edge == HAL_INTPIN_EDGE_LO2HI)
                {
                    p_btn->intpin_handler(p_btn->pin_no, HAL_INTPIN_EDGE_LO2HI);
                }
            }
            else if (action == NRF_GPIOTE_POLARITY_TOGGLE)
            {
                if (p_btn->trigger_edge == HAL_INTPIN_EDGE_BOTH)
                {
                    p_btn->intpin_handler(p_btn->pin_no, HAL_INTPIN_EDGE_BOTH);
                }
            }
            break;
        }
    }
}

uint32_t hal_intpin_init(hal_intpin_cfg_t *             p_intpins,
                         uint8_t                        intpin_count)
{
    uint32_t err_code = 0;

    // Save configuration.
    mp_intpins          = p_intpins;
    m_intpin_count      = intpin_count;
    
    if (!nrf_drv_gpiote_is_init())
    {
        err_code = nrf_drv_gpiote_init();
        if (err_code != NRF_SUCCESS)
        {
            return err_code;
        }
    }

    while (intpin_count--)
    {
        hal_intpin_cfg_t * p_pin = &p_intpins[intpin_count];

        if (p_pin->trigger_edge == HAL_INTPIN_EDGE_HI2LO)
        {
            nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_HITOLO(false); //true will open 16mhz crystal
            config.pull = p_pin->enable_pull_cfg;
            
            err_code = nrf_drv_gpiote_in_init(p_pin->pin_no, &config, gpiote_event_handler);
            
            if (err_code != NRF_SUCCESS)
            {
                return err_code;
            }
        }
        else if (p_pin->trigger_edge == HAL_INTPIN_EDGE_LO2HI)
        {
            nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(false); //true will open 16mhz crystal
            config.pull = p_pin->enable_pull_cfg;
            
            err_code = nrf_drv_gpiote_in_init(p_pin->pin_no, &config, gpiote_event_handler);
            
            if (err_code != NRF_SUCCESS)
            {
                return err_code;
            }
        }
        else
        {
            nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(false); //true will open 16mhz crystal
            config.pull = p_pin->enable_pull_cfg;
            
            err_code = nrf_drv_gpiote_in_init(p_pin->pin_no, &config, gpiote_event_handler);
            
            if (err_code != NRF_SUCCESS)
            {
                return err_code;
            }
        }
    }

    return NRF_SUCCESS;
}

uint32_t hal_intpin_enable(void)
{
    ASSERT(mp_intpins);

    uint32_t i;
    for (i = 0; i < m_intpin_count; i++)
    {
        nrf_drv_gpiote_in_event_enable(mp_intpins[i].pin_no, true);
    }

    return NRF_SUCCESS;
}

uint32_t hal_intpin_disable(void)
{
    ASSERT(mp_intpins);

    uint32_t i;
    for (i = 0; i < m_intpin_count; i++)
    {
       nrf_drv_gpiote_in_event_disable(mp_intpins[i].pin_no);
    }

    return NRF_SUCCESS;
}

void hal_intpin_enable_pin(uint32_t pin)
{
    uint32_t i;
    hal_intpin_cfg_t* p_intpin_cfg = NULL;
    
    for (i = 0; i < m_intpin_count; i++)
    {
        p_intpin_cfg = &mp_intpins[i];
        if (p_intpin_cfg->pin_no == pin)
        {
            nrf_gpio_cfg_input(p_intpin_cfg->pin_no, p_intpin_cfg->enable_pull_cfg);
            nrf_drv_gpiote_in_event_enable(p_intpin_cfg->pin_no, true);
            break;
        }
    }
}

void hal_intpin_disable_pin(uint32_t pin)
{
    uint32_t i;
    hal_intpin_cfg_t* p_intpin_cfg = NULL;
    
    for (i = 0; i < m_intpin_count; i++)
    {
        p_intpin_cfg = &mp_intpins[i];
        if (p_intpin_cfg->pin_no == pin)
        {
            nrf_gpio_cfg_input(p_intpin_cfg->pin_no, p_intpin_cfg->disable_pull_cfg);
            nrf_drv_gpiote_in_event_disable(p_intpin_cfg->pin_no);
            break;
        }
    }
}

